use bitflags::*;

bitflags!{
    ///CloneFlags
    pub struct CloneFlags: usize{
        ///默认信号
        const SIGCHLD = 17;
        ///设定子进程pid
        const MANUAL_SELECT_PID = 1;
        ///复制父进程pid
        const CLONE_CHILD_SETTID = 0x2000000;                                     
    }
}

///结构体：进程已执行的秒数和毫秒数
pub struct Timespec {
    ///结构体：进程已执行的秒数
    pub sec: usize,
    ///结构体：进程已执行的毫秒数
    pub usec: usize,
}

///Timespec结构的实现
impl Timespec {
    ///析构函数
    pub fn new() -> Self {
        Timespec { sec: 0, usec: 0 }
    }
}

///结构体：进程已执行的秒数
pub struct Tms {
    ///进程已执行的秒数
    pub sec: usize,
}

///Tms结构的实现 
impl Tms {
    ///析构函数
    pub fn new() -> Self {
        Tms { sec: 0 }
    }
}
///系统信息数据
#[allow(unused)]
pub struct Utsname{
    sysname: [u8; 65],                
    nodename: [u8; 65],
    release: [u8; 65],
    version: [u8; 65],
    machine: [u8; 65],
    domainname: [u8; 65],
}

impl Utsname{
    ///析构函数
    pub fn new() -> Self{
        Self{
            //sysname: utsname::str2u8("OurOS"),
            //nodename: utsname::str2u8("OurOS"),
            //release: utsname::str2u8("5.10.0-7-riscv64"),
            //version: utsname::str2u8("1.1"),
            //machine: utsname::str2u8("RISC-V64"),
            //domainname: utsname::str2u8("TCoreTEAM/OurOS"),
            sysname: Utsname::str2u8("Linux"),
            nodename: Utsname::str2u8("debian"),
            release: Utsname::str2u8("5.10.0-7-riscv64"),
            version: Utsname::str2u8("#1 SMP Debian 5.10.40-1 (2021-05-28)"),
            machine: Utsname::str2u8("riscv64"),
            domainname: Utsname::str2u8(""),
        }
    }
    ///str
    fn str2u8(str: &str) -> [u8;65]{
        let mut arr:[u8;65] = [0;65];
        let str_bytes = str.as_bytes();
        let len = str.len();
        for i in 0..len{
            arr[i] = str_bytes[i];
        }
        arr
    }
    ///bytes
    pub fn as_bytes(&self) -> &[u8] {
        let size = core::mem::size_of::<Self>();
        unsafe {
            core::slice::from_raw_parts(
                self as *const _ as usize as *const u8,
                size,
            )
        }
    }
}