//! RISC-V timer-related functionality

use crate::config::CLOCK_FREQ;
use crate::sbi::set_timer;
use log::trace;
use riscv::register::time;
use alloc::{collections::BTreeMap, sync::Arc, vec::Vec};
const TICKS_PER_SEC: usize = 100;
const MSEC_PER_SEC: usize = 1000;
const USEC_PER_SEC: usize = 1000_000;


///获取进程已执行的秒数
pub fn get_times(mut tms: Vec<*mut usize>) -> isize {
    let t = time::read();
    unsafe {
        *tms[0] = t / CLOCK_FREQ;
    }
    0
}

///进程已执行的秒数和毫秒数
pub fn get_time_with_timespec(mut ts: Vec<*mut usize>, _tz: usize) -> isize {
    let t = time::read();
    unsafe {
        *ts[0] = t / CLOCK_FREQ;
        *ts[1] = (t % CLOCK_FREQ) * 1000000 / CLOCK_FREQ;
        trace!("t {} sec {} usec {}", t, *ts[0], *ts[1]);
    }

    //The time will not be 0
    unsafe{
        if *ts[0] == 0 && *ts[1] == 0 {
            return -1;
        }  
    }
    0
}
///the sum of the time
pub fn sum_time(t_sec:usize, t_usec:usize, f_sec:usize, f_usec:usize) -> (usize,usize){
    let mut sum_usec = t_usec + f_usec;
    let mut sum_sec = t_sec + f_sec;
    if sum_usec >= USEC_PER_SEC{
        sum_usec -= USEC_PER_SEC;
        sum_sec += 1;
    } 
    (sum_sec, sum_usec)
}
///compare the time
pub fn compare_time(t_sec:usize, t_usec:usize, f_sec:usize, f_usec:usize) -> bool{
    // Compare sec
    if t_sec > f_sec{
        return true;
    }
    else if t_sec < f_sec{
        return false;
    } 
    //Compare usec
    if t_usec > f_usec{
        return true;
    }
    else if t_sec < f_usec{
        return false;
    } 
    return true;
}
///get current time
pub fn get_time() -> usize {
    time::read()
}

/// get usec time
pub fn get_time_us() -> usize {
    get_time() / (CLOCK_FREQ / USEC_PER_SEC) 
}
/// get CLOCL_FREQ
pub fn get_time_s() -> usize {
    get_time() / CLOCK_FREQ
}
/// get current time in microseconds
pub fn get_time_ms() -> usize {
    time::read() / (CLOCK_FREQ / MSEC_PER_SEC)
}
/// set the next timer interrupt
pub fn set_next_trigger() {
    set_timer(get_time() + CLOCK_FREQ / TICKS_PER_SEC);
}